import { EventBarModes } from "../../interfaces/frontend"
import Item from "./Item"
import { GameSettings } from "@components/Lobby/SettingsFrame/Setting"
import {
  faArrowRightFromBracket,
  faBroomWide,
  faComments,
  faEye,
  faEyeSlash,
  faGlasses,
  faPalette,
  faReply,
  faScribble,
  faSparkles,
  faSwords,
} from "@fortawesome/pro-solid-svg-icons"
import { useDrawProps } from "@hooks/useDrawProps"
import { useGameProps } from "@hooks/useGameProps"
import { socket } from "@lib/socket"
import { GamePropsSchema } from "@lib/zodSchemas"
import {
  Dispatch,
  SetStateAction,
  useCallback,
  useEffect,
  useMemo,
  useState,
} from "react"

export function setGameSetting(
  payload: GameSettings,
  setSetting: (settings: GameSettings) => string | null,
  full: (payload: GamePropsSchema) => void
) {
  return () => {
    const hash = setSetting(payload)
    socket.emit("gameSetting", payload, (newHash) => {
      if (newHash === hash) return
      console.log("hash", hash, newHash)
      socket.emit("update", full)
    })
  }
}

function EventBar({
  props: { setMode, clear },
}: {
  props: {
    setMode: Dispatch<SetStateAction<number>>
    clear: () => void
  }
}) {
  const [menu, setMenu] = useState<keyof EventBarModes>("main")
  const { shouldHide, color } = useDrawProps()
  const { payload, setSetting, full, setTarget } = useGameProps()

  const gameSetting = useCallback(
    (payload: GameSettings) => setGameSetting(payload, setSetting, full),
    [full, setSetting]
  )

  const items = useMemo<EventBarModes>(
    () => ({
      main: [
        {
          icon: "burger-menu",
          text: "Menu",
          callback: () => {
            setMenu("menu")
          },
        },
        {
          icon: faSwords,
          text: "Attack",
          callback: () => {
            setMenu("attack")
          },
        },
        {
          icon: "pen",
          text: "Draw",
          callback: () => {
            setMenu("draw")
          },
        },
        {
          icon: "gear",
          text: "Settings",
          callback: () => {
            setMenu("settings")
          },
        },
      ],
      menu: [
        {
          icon: faReply,
          text: "Return",
          iconColor: "#555",
          callback: () => {
            setMenu("main")
          },
        },
        {
          icon: faArrowRightFromBracket,
          text: "Leave",
          iconColor: "darkred",
          callback: () => {
            // router.push()
          },
        },
      ],
      attack: [
        {
          icon: faReply,
          text: "Return",
          iconColor: "#555",
          callback: () => {
            setMenu("main")
          },
        },
        {
          icon: "radar",
          text: "Radar scan",
          amount: 1,
          callback: () => {
            setMode(0)
            setTarget((e) => ({ ...e, show: false }))
          },
        },
        {
          icon: "torpedo",
          text: "Fire torpedo",
          amount: 1,
          callback: () => {
            setMode(1)
            setTarget((e) => ({ ...e, show: false }))
          },
        },
        {
          icon: "scope",
          text: "Fire missile",
          callback: () => {
            setMode(3)
            setTarget((e) => ({ ...e, show: false }))
          },
        },
      ],
      draw: [
        {
          icon: faReply,
          text: "Return",
          iconColor: "#555",
          callback: () => {
            setMenu("main")
          },
        },
        { icon: faBroomWide, text: "Clear", callback: clear },
        { icon: faPalette, text: "Color", iconColor: color },
        {
          icon: shouldHide ? faEye : faEyeSlash,
          text: shouldHide ? "Show" : "Hide",
          callback: () => {
            useDrawProps.setState({ shouldHide: !shouldHide })
          },
        },
      ],
      settings: [
        {
          icon: faReply,
          text: "Return",
          iconColor: "#555",
          callback: () => {
            setMenu("main")
          },
        },
        {
          icon: faGlasses,
          text: "Spectators",
          disabled: !payload?.game?.allowSpectators,
          callback: gameSetting({
            allowSpectators: !payload?.game?.allowSpectators,
          }),
        },
        {
          icon: faSparkles,
          text: "Specials",
          disabled: !payload?.game?.allowSpecials,
          callback: gameSetting({
            allowSpecials: !payload?.game?.allowSpecials,
          }),
        },
        {
          icon: faComments,
          text: "Chat",
          disabled: !payload?.game?.allowChat,
          callback: gameSetting({ allowChat: !payload?.game?.allowChat }),
        },
        {
          icon: faScribble,
          text: "Mark/Draw",
          disabled: !payload?.game?.allowMarkDraw,
          callback: gameSetting({
            allowMarkDraw: !payload?.game?.allowMarkDraw,
          }),
        },
      ],
    }),
    [
      clear,
      color,
      gameSetting,
      payload?.game?.allowChat,
      payload?.game?.allowMarkDraw,
      payload?.game?.allowSpecials,
      payload?.game?.allowSpectators,
      setMode,
      setTarget,
      shouldHide,
    ]
  )

  useEffect(() => {
    useDrawProps.setState({ enable: menu === "draw" })
  }, [menu])

  return (
    <div className="event-bar">
      {items[menu].map((e, i) => (
        <Item key={i} props={e} />
      ))}
    </div>
  )
}

export default EventBar
